
#include <QtGui>

#include "CorteoUi.h"
#include "CorteoParamWidget.h"
#include "techniques.h"
#include "constants.h"

#include "XYQt/FSDoubleValidator.h"
#include "XYQt/colorseries.h"

CorteoUi::CorteoUi(QWidget *parent, int maxlayer, int maxelement, QSplashScreen * splash)
       : QMainWindow(parent)
{

  qApp->setWindowIcon(QIcon(":/images/Corteo.png") );

  // create graphs and tab containing them
  tab = new QTabWidget(this);
  setCentralWidget(tab); // tab is the central widget

  mainGraph = new FSGraph("corteo.cgr",tab);
  angleGraph = new FSGraph("angle.cgr",tab);
  angleGraph->x->title = QString("Depth [A]");
  angleGraph->y->title = QString("Angle to detector axis [deg.]");

  tab->addTab(mainGraph, "Spectra/profiles");
  tab->addTab(angleGraph, "Angle to detector");
  tab->setTabEnabled (1, false);

  // create a parameter toolbox
  paramUi = new CorteoParamWidget(this, maxlayer, maxelement, splash);

  // initialize progress dialog pointer
  progress = NULL;

  // toolbox size defines dock size, a bit larger though
  QSize s = paramUi->toolBox->size();
  int w = s.width()+5;
  int h = s.height();
  paramUi->setFixedWidth(w+5);
  paramUi->setMinimumHeight(h);

  // create an unclosable dock widget that holds the parameter toolbox
  QDockWidget *dock = new QDockWidget("Parameters", this);
  dock->setAllowedAreas(Qt::LeftDockWidgetArea | Qt::RightDockWidgetArea);
  dock->setWidget(paramUi);
  addDockWidget(Qt::RightDockWidgetArea, dock);
  dock->setFixedWidth(w);
  dock->setFeatures(QDockWidget::QDockWidget::DockWidgetMovable | QDockWidget::DockWidgetFloatable);

  // graph will appear square
  resize(h+w,h); 

  connect( paramUi->pushButtonGo,           SIGNAL(clicked()),   SLOT(go())              );
  connect( qApp, SIGNAL(lastWindowClosed()), paramUi,  SLOT(saveParam()));
}


// rather complex function that loops through layers and elements to find which have to be simulated
// then call runCorteo() for each, and create datasets from the results
bool CorteoUi::go() {

  bool isError = false;

  // count the number of simulations to carry out
  int numberSim = 0;
  bool isUsedElement = false;
  for(int ilayer=0; ilayer<paramUi->numberLayer(); ilayer++) {
    for(int ielement=0; ielement<paramUi->numberElement(ilayer); ielement++) {
      if(paramUi->isUsedElement(ilayer, ielement)) {
        isUsedElement = true;
      }
      if(paramUi->isSimulatedElement(ilayer, ielement)) {
        numberSim++;
      }
    }
  }

  // anything to simulate?
  if(!isUsedElement ) {
    QMessageBox::warning( this, "CorteoUi", "No used element.");
    return false;
  }
  if(numberSim==0 && paramUi->technique()>0 ) {
    QMessageBox::warning( this, "CorteoUi", "No element selected for simulation.\n(Check 'Simulate this element' boxes in layers.)");
    return false;
  }
  if(paramUi->lineEditCharge->text().toDouble()==0) {
    QMessageBox::warning( this, "CorteoUi", "Incident charge is 0.");
    return false;    
  }

  // prepare for simulations: disable 'go' button and create a progress bar
  paramUi->pushButtonGo->setEnabled(false);
  progress = new QProgressDialog(this);
  progress->setCancelButton ( new QPushButton ("Cancel", progress) );

  // open log file
  QFile logfile("CorteoUi.log");
  if (!logfile.open(QIODevice::WriteOnly | QIODevice::Text | QIODevice::Append)) {
    QMessageBox::warning( this, "CorteoUi", "Can't open 'CorteoUi.log'.");
    return false;
  }

  if(paramUi->technique()<0) {
    
    // implantation, only one simulation to run
    double q = paramUi->lineEditCharge->text().toDouble();
    if(runCorteo(0,0,0,1)) {

      // simulation successful: create a dataset, load resulting profile into it, and add it to the graph
      FSDataSet * d = new FSDataSet( paramUi->beam().symbol, QPen(colorSeries(mainGraph->getndatasets()), 1) );
      if(d->loadfile("corteo.out",1,2,1)==false) { // read column 1 (depth) as x, column 2 as y, skipping line 1
        QMessageBox::warning( this, "CorteoUi", "Can't open resulting spectrum file corteo.out. Stopping here.");
        endSimulations();
        logfile.close();
        return false;
      }

      for(int i=0; i<d->getndata(); i++) 
        d->yp[i]*=q*1e8; // at/A/ion to [at/cm3]/[at/cm2] * [at/cm2]
      mainGraph->appendDataSet(d);
      tab->setTabEnabled (1, false);

    }

    // in case of full cascade, also load displacements profiles
    // (but Corteo generates only one file per element, event if present in many layer, so load them only once)
    if(paramUi->technique()==TECH_IMPREC) {
      QList<int> Z;
      // loop through all layers and elements 
      for(int ilayer=0; ilayer<paramUi->numberLayer(); ilayer++) {
        for(int ielement=0; ielement<paramUi->numberElement(ilayer); ielement++) {
          // if element is used
          if(paramUi->isUsedElement(ilayer, ielement)) {
            Atom a = paramUi->element(ilayer, ielement);
            if(!Z.contains(a.Z)) {
            
              FSDataSet * d = new FSDataSet( QString("%1 disp.").arg(a.symbol), QPen(colorSeries(mainGraph->getndatasets()), 1) );
              QString filename = QString("impl%1.out").arg(a.Z,3,10,QChar('0'));
              if(d->loadfile(filename,1,2,1)==false) { // read column (energy) 1 as x, column 2 as y, skipping line 1
                QMessageBox::warning( this, "CorteoUi", QString("Can't open file %1. Stopping here.").arg(filename));
                endSimulations();
                logfile.close();
                return false;
              } 
              for(int i=0; i<d->getndata(); i++) 
                d->yp[i]*=q*1e8; // at/A/ion to [at/cm3]/[at/cm2] * [at/cm2]
              mainGraph->appendDataSet(d);
              Z.append(a.Z);
            }
          } 
        }
      }
    }
    
    // append the content of corteo.detect to the list file
    if(paramUi->checkBoxListMode->isChecked()) {
      QFile fi("corteo.details");
      QFile fo(paramUi->listFile);
      QMessageBox::warning( this, "CorteoUi", paramUi->listFile);

      if(fi.open(QIODevice::ReadOnly | QIODevice::Text) && 
         fo.open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text)) {

        QTextStream in(&fi);
        QTextStream out(&fo);
        while (!in.atEnd()) {
          QString line = in.readLine();
          out << line << endl;
        }
        fi.close();
        fo.close();
      }
    }

    QTextStream logstream(&logfile);
    QDateTime dt = QDateTime::currentDateTime();
    logstream << "======= " << dt.toString("yyyy/MM/dd hh:mm:ss") << " =======" << endl;
    logstream << corteoOutput << endl;
    if(!corteoError.isEmpty()) {
      logstream << "--- Warning and Error(s) ---" << endl;
      logstream << corteoError << endl ;
      isError = true;
    }
    logstream << endl;
    
    if(q==1) {
      // fluence not set, normalized depth profile
      mainGraph->y->title = QString("Concentration/fluence [at/cm^{3}]/[at/cm^{2}]");
    } else {
      // fluence set to some value
      mainGraph->y->title = QString("Concentration [at/cm^{3}]");
    }
    mainGraph->x->title = QString("Depth [A]");


  } else { // IBA

    // check if listfile is requested and, if it already exists, should be replaced
    if(paramUi->checkBoxListMode->isChecked() && QFile::exists(paramUi->listFile)) {
      if(QMessageBox::question(this, "CorteoUi", 
                               QString("%1 already exist. Replace?").arg(paramUi->listFile),
                               QMessageBox::Yes, QMessageBox::No )==QMessageBox::Yes) {
        QFile::remove(paramUi->listFile);
      }
    }

    // in case user wants to sum same detected element in one spectrum (doSum = true),
    // create lists of Z of elements already simulated and there corresponding dataset
    int iSim = 0;
    bool doSum = paramUi->checkBoxSumElements->isChecked();

    QList<int> Zlist;  // list of already simulated Z
    QList<FSDataSet *> spectrumList; // list of datasets containing the spectra of already simulated elements
    QList<FSDataSet *> angleDataList; // list of datasets containing angle-to-detector of already simulated elements

    bool isBeam = paramUi->isBeamDetectionTechnique();               // true if the spectrum is made of scattered beam atoms
    Atom b = paramUi->beam();                                        // beam atom
    double q = paramUi->lineEditCharge->text().toDouble()/ELEMENTARY_CHARGE; // number of incident atoms

    // loop through all layers and elements 
    for(int ilayer=0; ilayer<paramUi->numberLayer(); ilayer++) {
      for(int ielement=0; ielement<paramUi->numberElement(ilayer); ielement++) {

        // is this an active element the user wants to be simulated?
        if(paramUi->isSimulatedElement(ilayer, ielement)) {

          // print timestamp in logfile
          QTextStream logstream(&logfile);
          QDateTime dt = QDateTime::currentDateTime();
          logstream << "======= " << dt.toString("yyyy/MM/dd hh:mm:ss") << " =======" << endl;
          logstream << QString("Simulation of element %1 in layer %2:\n").arg(ielement).arg(ilayer) << endl;

          // run simulation for this element in this layer
          bool success = runCorteo(ilayer,ielement, iSim, numberSim);

          // append standard output to logfile, as well as error output if any
          logstream << corteoOutput << endl;
          if(!corteoError.isEmpty()) {
            logstream << "--- Warning and Error(s) ---" << endl;
            logstream << corteoError << endl;
            isError = true;
          }
          logstream << endl;

          // simulation successful?
          if(success) {

            double fraction = paramUi->fraction(ilayer, ielement);
            Atom a = paramUi->element(ilayer, ielement);
            
            if(!isBeam) b=a;  // detected atom: element itself if isBeamDetectionTechnique==false
            // (hence, if user wants a sum, in case of RBS, all spectra will be summed as the detected particle is
            //  undistinguishable, while in case of ERD, only the the spectra from the same detected element will be summed)

            // if user wants to sum simulations  from the same Z and if the element was already simulated 
            // in a previous layer, load the data and add them to the existing dataset
            if(doSum && Zlist.contains(b.Z)) {
              // temporarily create a dataset and load the simulated spectrum into it
              FSDataSet * d = new FSDataSet();
              if(d->loadfile("corteo.out",1,2,1)==false) { // read column (energy) 1 as x, column 2 as y, skipping line 1
                QMessageBox::warning( this, "CorteoUi", "Can't open resulting spectrum file corteo.out. Stopping here.");
                endSimulations();
                logfile.close();
                return false;
              }

              // add the simulated spectrum to the already existing spectrum
              FSDataSet * p = spectrumList[Zlist.indexOf(b.Z)];
              for(int i=0; i<p->getndata(); i++) 
                p->yp[i] += d->yp[i]*q*fraction;

              delete d; // delete the temporary spectrum

              if(paramUi->checkBoxGraphAngle->isChecked()) {
                // temporarily create a dataset and load content of corteo.angle file
                FSDataSet * da = new FSDataSet();
                if(da->loadfile("corteo.angle")==false) {
                  QMessageBox::warning( this, "CorteoUi", "Can't open angle-to-detector file corteo.angle. Stopping here.");
                  endSimulations();
                  logfile.close();
                  return false;
                } 

                // add the angle data to the already existing data for this element
                FSDataSet * pa = angleDataList[Zlist.indexOf(b.Z)];
                pa->grow(da->getndata());
                for(int i=0; i<da->getndata(); i++) 
                  pa->append(da->xp[i],da->yp[i]);

                delete da; // delete the temporary angle dataset
              }

            } else {
              // no spectrum sumation, or element was not already in the list,

              // dataset name 
              QString name = QString("%1 %2").arg(a.symbol).arg(ilayer); // !isBeam: element symbol + layer number
              if(doSum) {
                if(isBeam) name = paramUi->beam().symbol;
                else name = a.symbol;
              }

              // create a dataset, load the spectrum into it and append the dataset to spectrumList
              FSDataSet * d = new FSDataSet( name, QPen(colorSeries(iSim), 1) );
              if(d->loadfile("corteo.out",1,2,1)==false) { // read column (energy) 1 as x, column 2 as y, skipping line 1
                QMessageBox::warning( this, "CorteoUi", "Can't open resulting spectrum file corteo.out. Stopping here.");
                endSimulations();
                logfile.close();
                return false;
              } 
              for(int i=0; i<d->getndata(); i++) 
                d->yp[i]*=q*fraction; // multiply by charge and fraction of this element in this layer
              spectrumList.append(d); // append to the list of datasets to add to the main graph

              if(paramUi->checkBoxGraphAngle->isChecked()) {
                // create a dataset with content of corteo.angle file
                FSDataSet * da = new FSDataSet( name, QPen(), 11, 1, QPen(colorSeries(iSim), 1));
                if(da->loadfile("corteo.angle")==false) {
                  QMessageBox::warning( this, "CorteoUi", "Can't open angle file corteo.angle. Stopping here.");
                  endSimulations();
                  logfile.close();
                  return false;
                }
                angleDataList.append(da); // append to the list of datasets to add to the main graph
              }

              // also append element to the list of already existing element spectra 
              Zlist.append(b.Z);
            }

          } else {
            // there was an important error preventing simulations to continue
            // or user killed the simulation
            endSimulations();
            logfile.close();
            return false;
          }
          iSim++;
        }
      }
    }

    // append all spectra in spectrumList to the graph
    for(int i=0; i<spectrumList.count(); i++)
      mainGraph->appendDataSet(spectrumList[i]);

    if(q==1) {
      // charge not set, normalized spectrum
      mainGraph->y->title = QString("counts/coulomb");
    } else {
      // charge set to some value
      mainGraph->y->title = QString("counts");
    }
    mainGraph->x->title = QString("Energy [eV]");


    if(angleGraph->getndatasets()>0) {
      tab-> setCurrentIndex(1);
      if(QMessageBox::question(this, "CorteoUi", "This graph contains data from previous simulation. Clear them?",
                               QMessageBox::Yes, QMessageBox::No )==QMessageBox::Yes) {
          angleGraph->clearDataSets();
      }
    }

    if(paramUi->checkBoxGraphAngle->isChecked()) {
    // user requested an angle-to-detector graph, append all dataset to the graph and show it

      // append angle datasets
      for(int i=0; i<angleDataList.count(); i++) {
        FSDataSet * d = angleDataList[i];
        angleGraph->appendDataSet(d);

        // generate a curve delimiting the 99th percentile of angles to detector
        // we will generate 10 such points, each for 1/10th of the data
        // as we need the 99th percentile 10 times, we need at least 1000 points
        d->sort(); // in depth order
        int n = d->getndata()/10;
        FSDataSet * percentile99 = new FSDataSet(QString("%1 99%").arg(d->getname()), QPen(d->symbolPen.color().light(80), 2));
        if(n>100) {
          FSDataSet * temp = new FSDataSet();
          temp->grow(n);
          double xmoy = 0;
          for(int j=0; j<10; j++) {
            temp->clear();
            // create a dataset containing a subset (1/10) of the points
            for(int k=j*n; k<(j+1)*n; k++) {
              temp->append(d->xp[k],d->yp[k]);
              xmoy += d->xp[k];
            }
            xmoy /= n;

            // swap x and y so we can use the sort function on y data
            temp->swap();
            temp->sort();

            // add the point that is the 99th/100 of the sorted data
            percentile99->append(xmoy,temp->xp[int(temp->getndata()*0.99+.5)]);
          }
          angleGraph->appendDataSet(percentile99);
        }
      }
      tab->setTabEnabled(1, true);
    } else {
      // no angle data, disable tab
      if(angleGraph->getndatasets()==0) {
        tab-> setCurrentIndex(0);
        tab->setTabEnabled(1, false);
      }
    }
  }
  endSimulations();
  logfile.close();

  if(isError) {
    QMessageBox::warning(this,"Corteo","There were warnings or errors, please check log.");
    return false;
  }
  return true; 
}


void CorteoUi::endSimulations() {
  delete progress;
  progress = NULL;
  paramUi->pushButtonGo->setEnabled(true);
}



// function that run a Corteo simulation
//   in the case of, each element in each layer is simulated in a separate simulation
//   so the the element 'ielement' in layer 'ilayer' is specified, 
//   as well this simulation number iSim and the total number of simulation numSim for the progress notification
// 
//   in the case of implantation, the parameters are useless since there is only one simulation 
// 
bool CorteoUi::runCorteo(int ilayer, int ielement, int iSim, int numSim ) {

  // get the corteo input parameters
  bool error;
  QString s = paramUi->corteo(ilayer, ielement, &error);
  if(error) {
    // something's wrong, stop here
    QMessageBox::warning(this, "CorteoUi", s);
    return false; 
  }

  // write them to corteo.in
  QFile file( "corteo.in" );
  if( !file.open( QIODevice::WriteOnly ) ) {
    QMessageBox::warning( 0, "CorteoUi", "Can't open 'corteo.in'.");
    return false;
  }
  QTextStream ts( &file );
  ts << s << endl;
  file.close();

  // remove old corteo.out and corteo.angle so we are not confused reading old results
  QFile::remove("corteo.out");
  if(paramUi->checkBoxGraphAngle->isChecked()) 
    QFile::remove("corteo.angle");

  // create a process to run corteo
  QProcess corteoProcess( this );
#ifdef WIN32
  corteoProcess.start("corteo.exe");
#else
  corteoProcess.start("./corteo");
#endif

  // check if corteo started (can be long if computer is busy, default waits up to 30 sec.)
  if(!corteoProcess.waitForStarted()) {
    corteoProcess.kill();
    QMessageBox::warning( this, "CorteoUi", "Can't start corteo.");
    return false;
  }

  // process other events
  qApp->processEvents();

  // wait up to 3 second to see if corteo finished
  bool finished = corteoProcess.waitForFinished(3000);
  corteoOutput  = corteoProcess.readAllStandardOutput();
  corteoError   = corteoProcess.readAllStandardError();

  // if computer is slow, it may not have finish initializing matrices after 3 seconds, so wait longer
  while (corteoOutput.isEmpty() && corteoError.isEmpty()) {
    corteoProcess.waitForFinished(1000);
    corteoOutput  += corteoProcess.readAllStandardOutput();
    corteoError   += corteoProcess.readAllStandardError();
  }

  if(!finished) {
    // find number of threads
    int i = corteoOutput.indexOf("thread");
    int numthreads = i>0?corteoOutput.left(i).toInt():0;

    if(numthreads<1) {
      QMessageBox::warning( 0, "Corteo error", "Can't find number of threads");
      return false;
    }

    // not finished, fire up a progress bar
    progress->setLabelText( QString("Corteo simulation %1 of %2").arg(iSim+1).arg(numSim) );
    progress->setRange ( 0, numSim*10*numthreads );

    while(!finished) {

      // check every 1/4 second if corteo finished, and process other events
      finished = corteoProcess.waitForFinished(250);
      qApp->processEvents();

      corteoOutput += corteoProcess.readAllStandardOutput();
      corteoError  += corteoProcess.readAllStandardError();

      // to know how far we are in the simulation, we need to count the dots after "Simulation running"
      // (at the end of simulation, dots will be 10 times the number of threads)
      QString sr = "Simulation running";
      i = corteoOutput.indexOf(sr)+sr.length();
      progress->setValue(iSim*10*numthreads  + corteoOutput.length()-i);
 
      if(progress->wasCanceled()) {
        QMessageBox::warning(this, "corteo", "Killing corteo simulation...");
        corteoProcess.kill();
        return false;
      }
    }
  }
  return true;
}

