/* This file is part of the Allegria project
   Copyright (C) 2004 Francois Schiettekatte <francois.schiettekatte@umontreal.ca>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

// Dialog for editing axis properties

#include <QPen>
#include <QValidator>
#include <QColorDialog>
#include <QFontDialog>

#include "FSEditAxis.h"
#include "FSDoubleValidator.h"

#include "xpmLineStyle.h"

FSEditAxis::FSEditAxis(FSAxis * axis, QWidget *parent, QString name )
            : QDialog(parent)
{

    QIntValidator * validatorInt = new QIntValidator(2,99, this);
    FSDoubleValidator * fsdvalidator = new FSDoubleValidator(this);

    setupUi(this);

    selectedTitleFont  = axis->titleFont;
    selectedLabelFont  = axis->labelFont;
    selectedTitleColor = axis->titleColor;
    selectedLabelColor = axis->labelColor;

    selectedFrameColor = axis->framePen.color();
    selectedTickColor  = axis->tickPen.color();
    selectedGridColor  = axis->gridPen.color();

    frameLineStyleBox->addItem(QIcon(lsNone_xpm),"none");
    gridLineStyleBox->addItem(QIcon(lsNone_xpm),"none");

    frameLineStyleBox->addItem(QIcon(lsSolid_xpm),"solid");
    gridLineStyleBox->addItem(QIcon(lsSolid_xpm),"solid");

    frameLineStyleBox->addItem(QIcon(lsDash_xpm),"dot");
    gridLineStyleBox->addItem(QIcon(lsDash_xpm),"dot");

    frameLineStyleBox->addItem(QIcon(lsDot_xpm),"dot");
    gridLineStyleBox->addItem(QIcon(lsDot_xpm),"dot");

    frameLineStyleBox->addItem(QIcon(lsDashDot_xpm),"dash-dot");
    gridLineStyleBox->addItem(QIcon(lsDashDot_xpm),"dash-dot");

    frameLineStyleBox->addItem(QIcon(lsDashDotDot_xpm),"dash-dot-dot");
    gridLineStyleBox->addItem(QIcon(lsDashDotDot_xpm),"dash-dot-dot");

    pm = QPixmap(20,20);

    // checkbox titles according to axis type
    QString pos1 = axis->isHorizontal?QString("Bottom"):QString("Left");
    QString pos2 = axis->isHorizontal?QString("Top"):QString("Right");

    // title options
    titleEdit->setText(axis->title);
    titleLeft->setChecked(axis->gettitlePos()&1);
    titleRight->setChecked(axis->gettitlePos()&2);
    titleLeft->setText(pos1);
    titleRight->setText(pos2);

    pm.fill(selectedTitleColor);
    btTitleColor->setIcon(QIcon(pm));

    connect( btTitleFont, SIGNAL(clicked()), SLOT(newTitleFont()) );
    connect( btTitleColor, SIGNAL(clicked()), SLOT(newTitleColor()) );

    // label options
    isLogAxis->setChecked(axis->logAxis);
    labelLeft->setChecked(axis->getlabelPos()&1);
    labelRight->setChecked(axis->getlabelPos()&2);
    labelLeft->setText(pos1);
    labelRight->setText(pos2);
    minEdit->setText(QString::number(axis->logAxis?pow(10.,axis->getmin()):axis->getmin()));
    minEdit->setValidator(fsdvalidator);
    maxEdit->setText(QString::number(axis->logAxis?pow(10.,axis->getmax()):axis->getmax()));
    maxEdit->setValidator(fsdvalidator);

    pm.fill(selectedLabelColor);
    btLabelColor->setIcon(QIcon(pm));

    connect( btLabelFont, SIGNAL(clicked()), SLOT(newLabelFont()) );
    connect( btLabelColor, SIGNAL(clicked()), SLOT(newLabelColor()) );

    // frame options
    frameLeft->setChecked(axis->getframePos()&1);
    frameRight->setChecked(axis->getframePos()&2);
    frameLeft->setText(pos1);
    frameRight->setText(pos2);

    frameLineWidthEdit->setText(QString::number(axis->framePen.width()));
    frameLineWidthEdit->setValidator(validatorInt);
    frameLineStyleBox->setCurrentIndex(axis->framePen.style());

    pm.fill(selectedFrameColor);
    frameColorbt->setIcon(QIcon(pm));
    connect( frameColorbt, SIGNAL(clicked()), SLOT(newFrameColor()) );

    // tick options
    tickInside->setChecked(axis->gettickPos()&1);
    tickOutside->setChecked(axis->gettickPos()&2);
    tickLineWidthEdit->setText(QString::number(axis->tickPen.width()));
    tickLineWidthEdit->setValidator(validatorInt);
    tickNumberEdit->setText(QString::number(axis->getntick()));
    tickNumberEdit->setValidator(validatorInt);

    pm.fill(selectedTickColor);
    tickColorbt->setIcon(QIcon(pm));
    connect( tickColorbt, SIGNAL(clicked()), SLOT(newTickColor()) );

    // grid options
    bool b = axis->grid;
    drawGridBox->setChecked(b);
    gridLineWidthEdit->setText(QString::number(axis->gridPen.width()));
    gridLineWidthEdit->setValidator(validatorInt);
    gridLineStyleBox->setCurrentIndex(axis->gridPen.style());
    
    pm.fill(selectedGridColor);
    gridColorbt->setIcon(QIcon(pm));
    connect( gridColorbt, SIGNAL(clicked()), SLOT(newGridColor()) );


}

QString FSEditAxis::title( void )
{
    return titleEdit->text();
}

int FSEditAxis::titlePos( void )
{
    int pos = 0;
    if(titleLeft->isChecked())pos+=1;
    if(titleRight->isChecked())pos+=2;
    return pos;
}

QFont FSEditAxis::titleFont( void )
{
    return selectedTitleFont;
}

QColor FSEditAxis::titleColor( void )
{
    return selectedTitleColor;
}


int FSEditAxis::labelPos( void )
{
    int pos = 0;
    if(labelLeft->isChecked())pos+=1;
    if(labelRight->isChecked())pos+=2;
    return pos;
}

QFont FSEditAxis::labelFont( void )
{
    return selectedLabelFont;
}

QColor FSEditAxis::labelColor( void )
{
    return selectedLabelColor;
}


QPen FSEditAxis::framePen( void )
{
    return QPen(QBrush(selectedFrameColor),frameLineWidthEdit->text().toInt(),Qt::PenStyle(frameLineStyleBox->currentIndex()));
}

int FSEditAxis::framePos( void )
{
    int pos = 0;
    if(frameLeft->isChecked())pos+=1;
    if(frameRight->isChecked())pos+=2;
    return pos;
}


QPen FSEditAxis::tickPen( void )
{
    return QPen(QBrush(selectedTickColor),tickLineWidthEdit->text().toInt());
}

int FSEditAxis::tickPos( void )
{
    int pos = 0;
    if(tickInside->isChecked())pos+=1;
    if(tickOutside->isChecked())pos+=2;
    return pos;
}

QPen FSEditAxis::gridPen( void )
{
    return QPen(QBrush(selectedGridColor),gridLineWidthEdit->text().toInt(),Qt::PenStyle(gridLineStyleBox->currentIndex()));
}

bool FSEditAxis::drawGrid( void )
{
    return drawGridBox->isChecked();
}


bool FSEditAxis::isLog( void )
{
    return isLogAxis->isChecked();
}

double FSEditAxis::min( void )
{
    return minEdit->text().toDouble();
}

double FSEditAxis::max( void )
{
    return maxEdit->text().toDouble();
}

bool FSEditAxis::extremaChanged( void )
{
   // min or max value were changed
    return (minEdit->isModified() || maxEdit->isModified());
}


int FSEditAxis::ntick( void )
{
    return tickNumberEdit->text().toInt();
}

void FSEditAxis::newColor(QColor * selectedColor, QPushButton * btColor)
{
    QColor c = QColorDialog::getColor(*selectedColor, this);
    if(c.isValid())
    {
        *selectedColor = c;
        pm.fill(*selectedColor);
        btColor->setIcon(QIcon(pm));
    }
}

void FSEditAxis::newTitleColor( void )
{
    newColor(&selectedTitleColor, btTitleColor);
}

void FSEditAxis::newLabelColor( void )
{
    newColor(&selectedLabelColor, btLabelColor);
}

void FSEditAxis::newFrameColor( void )
{
    newColor(&selectedFrameColor, frameColorbt);
}

void FSEditAxis::newTickColor( void )
{
    newColor(&selectedTickColor, tickColorbt);
}

void FSEditAxis::newGridColor( void )
{
    newColor(&selectedGridColor, gridColorbt);
}


void FSEditAxis::newTitleFont( void )
{
    bool ok;
    QFont f = QFontDialog::getFont(&ok, selectedTitleFont, this);
    if(ok)
        selectedTitleFont = f;
}

void FSEditAxis::newLabelFont( void )
{
    bool ok;
    QFont f = QFontDialog::getFont(&ok, selectedLabelFont, this);
    if(ok)
        selectedLabelFont = f;
}


