/*
 QRDecomposition class of Jama package partialy ported  from java to C++
 and modified to support only the particular case of a square matrix A
 and one dimensional B and X; by Francois Schiettekatte 2001/01/10
*/

/*
Copyright Notice from Jama web site (math.nist.gov/javanumerics/jama)

This software is a cooperative product of The MathWorks and the
National Institute of Standards and Technology (NIST) which has been
released to the public domain. Neither The MathWorks nor NIST assumes
any responsibility whatsoever for its use by other parties, and makes
no guarantees, expressed or implied, about its quality, reliability,
or any other characteristic.

The same applies to this port.
*/

/* QR Decomposition.

   For an m-by-n matrix A with m >= n, the QR decomposition is an m-by-n
   orthogonal matrix Q and an n-by-n upper triangular matrix R so that
   A = Q*R.

   The QR decompostion always exists, even if the matrix does not have
   full rank, so the constructor will never fail.  The primary use of the
   QR decomposition is in the least squares solution of nonsquare systems
   of simultaneous linear equations.  This will fail if isFullRank()
   returns false.
*/

#include <math.h>
#include "JamaQRDecomposition.h"

//------------------------
// Constructor
//------------------------
// QR Decomposition, computed by Householder reflections.
// param A    Square matrix
// set Rdiag  Structure to access R and the Householder vectors and compute Q.
QRDecomposition::QRDecomposition(double **qr, int nn)
{
    int i;

    // Initialize
    QR = qr;
    n = nn;
    Rdiag = new double[n];

    // Main loop.
    for (int k = 0; k < n; k++) {
        // Compute 2-norm of k-th column
        double nrm = 0.;
        for (i = k; i < n; i++)
            nrm = pow(nrm*nrm+QR[i][k]*QR[i][k],0.5);  //Maths.hypot(nrm,QR[i][k]);


        if (nrm != 0.0) {
            // Form k-th Householder vector.
            if (QR[k][k] < 0.)
               nrm = -nrm;

            for (i = k; i < n; i++)
               QR[i][k] /= nrm;

            QR[k][k] += 1.0;

            // Apply transformation to remaining columns.
            for (int j = k+1; j < n; j++) {
               double s = 0.0;
	       int ii;
               for (ii = k; ii < n; ii++)
                  s += QR[ii][k]*QR[ii][j];

               s = -s/QR[k][k];
               for (ii = k; ii < n; ii++)
                  QR[ii][j] += s*QR[ii][k];

            }
        }
        Rdiag[k] = -nrm;
    }
}


//------------------------
// Destructor
//------------------------
//  deallocate variables allocated by constructor
QRDecomposition::~QRDecomposition()
{
    delete[] Rdiag;
}


//------------------------
// Public Methods
//------------------------
// Is the matrix full rank?
// return     true if R, and hence A, has full rank.
//
bool QRDecomposition::isFullRank( void )
{
    for (int j = 0; j < n; j++)
        if (Rdiag[j] == 0.)
            return false;
    return true;
}

// Least squares solution of A*X = B
// param B    vector with as many rows as A.
// return     X that minimizes the two norm of Q*R*X-B.
//
double* QRDecomposition::solve(double *B)
{
	int i,k;
    double *X = new double[n];
    for (k = 0; k < n; k++) X[k]=B[k];

    // Compute Y = transpose(Q)*B
    for (k = 0; k < n; k++) {
        double s = 0.0;
        for (i = k; i < n; i++) {
            s += QR[i][k]*X[i];
        }
        s = -s/QR[k][k];
        for (i = k; i < n; i++) {
            X[i] += s*QR[i][k];
        }
    }

    // Solve R*X = Y;
    for (k = n-1; k >= 0; k--) {
        X[k] /= Rdiag[k];
        for (i = 0; i < k; i++) {
            X[i] -= X[k]*QR[i][k];
        }
    }
    return X;

}
