/*  
    Copyright 2007, Francois Schiettekatte

    This file is part of Corteo.

    Corteo is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Corteo is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Corteo.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "corteoindex.h"

// Xindex: compute an integer index using the binary representation of a floating point value val/MIN
// (X is E for reduced energy, S for reduced impact parameter or D for energy of stopping).
// These indexes are used to access sin/cos tables of the scattering angle and stopping power tables.
// The division by MIN is actually carried out by subtracting the appropriate BIAS from the resulting index
//
// Input: val, with val >= MIN, (MIN = MINE, MINS, or MIND)
//
// Returns an integer equal to 16 times the exponent base 2 of val 
// + 1 for each 1/16 interval between each power of 2
// (Index of input value = MIN is 0)

// examples
//       val/MIN    index
//       1           0
//       1.0625      1
//       1.0626      1
//       1.1249      1
//       1.1250      2
//       1.2         3
//       1.25        4
//       2           16
//       5           40
//
// BEWARE: architecture dependent!
// Assuming 32-bit 'unsigned long' integer and float
// Assuming IEEE Standard 754 representation for float:
//   bit 31: sign
//   bit 30-23: exponent (base 2), biased by 127 (i.e. value of exponent for 2^0 is 127)
//   bit 22-0: mantissa

// version that compute the index of a reduced energy 
unsigned long Eindex(float val) {
  unsigned long ll;
  ll = *(unsigned long *)&val;
  ll = (ll >> SHIFTE)-BIASE;

#ifdef INDEX_BOUND_CHECKING
  if(val<MINE) {
    EminErr++;
    return 0;
  }
  if(ll>=DIME) {
    EmaxErr++;
    return DIME-1;
  }
#endif

  return ll;
}

// version that compute the index of a reduced impact parameter
unsigned long Sindex(float val) {
  unsigned long ll;
  ll= *(unsigned long *)&val;
  ll = (ll >> SHIFTS)-BIASS;

#ifdef INDEX_BOUND_CHECKING
  if(val<MINS) {
    SminErr++;
    return 0;
  }
  if(ll>=DIMS) {
    SmaxErr++;
    return DIMS-1;
  }
#endif

  return ll;
}

// version that compute the index of the energy of a stopping power and enegy straggling
unsigned long Dindex(float val) {
  unsigned long ll;
  ll = *(unsigned long *)&val;
  ll = (ll >> SHIFTD)-BIASD;

#ifdef INDEX_BOUND_CHECKING
  if(val<MIND) {
    DminErr++;
    return 0;
  }
  if(ll>=DIMD) {
    DmaxErr++;
    return DIMD-1;
  }
#endif

  return ll;
}


// return the float value corresponding to index
// (Actually returns the average value between the value of this index and the next
//  because when an index is computed, values are truncated to the lower bound of the interval.)
float Eval(unsigned long index) {
  float temp1, temp2;
  unsigned long ll;
  
  ll = (index+BIASE)<<SHIFTE;
  temp1 = (*(float *)&ll);
  ll = (index+1+BIASE)<<SHIFTE;
  temp2 = (*(float *)&ll);

  return (temp1+temp2)*0.5f;
}

float Sval(unsigned long index) {
  float temp1, temp2;
  unsigned long ll;
  
  ll = (index+BIASS)<<SHIFTS;
  temp1 = (*(float *)&ll);
  ll = (index+1+BIASS)<<SHIFTS;
  temp2 = (*(float *)&ll);

  return (temp1+temp2)*0.5f;
}

float Dval(unsigned long index) {
  float temp1, temp2;
  unsigned long ll;
  
  ll = (index+BIASD)<<SHIFTD;
  temp1 = (*(float *)&ll);
  ll = (index+1+BIASD)<<SHIFTD;
  temp2 = (*(float *)&ll);

  return (temp1+temp2)*0.5f;
}


